@@ -133,7 +133,6 @@ class BackendZ3(Backend):
         self._op_raw['__lt__'] = self._op_raw_ULT
 
         self._op_raw['Reverse'] = self._op_raw_Reverse
-        self._op_raw['Identical'] = self._identical
         self._op_raw['fpToSBV'] = self._op_raw_fpToSBV
         self._op_raw['fpToUBV'] = self._op_raw_fpToUBV
 
@@ -253,13 +252,6 @@ class BackendZ3(Backend):
         self._simplification_cache_key.clear()
         self._simplification_cache_val.clear()
 
-    @condom
-    def _size(self, a):
-        if not isinstance(a, z3.BitVecRef) and not isinstance(a, z3.BitVecNumRef):
-            l.debug("Unable to determine length of value of type %s", a.__class__)
-            raise BackendError("Unable to determine length of value of type %s" % a.__class__)
-        return a.size()
-
     def _name(self, o): #pylint:disable=unused-argument
         l.warning("BackendZ3.name() called. This is weird.")
         raise BackendError("name is not implemented yet")
@@ -510,29 +502,33 @@ class BackendZ3(Backend):
         if append_children:
             args.extend(children)
 
-        # hmm.... honestly not sure what to do here
-        result_ty = op_type_map[z3_op_nums[decl_num]]
-        ty = type(args[-1])
+        if z3_op_nums[decl_num] == 'Z3_OP_INTERNAL' and not args:
+            # special case for Z3 string constants
+            a = self._string_constant_from_ast(ctx, ast, decl)
+        else:
+            # hmm.... honestly not sure what to do here
+            result_ty = op_type_map[z3_op_nums[decl_num]]
+            ty = type(args[-1])
 
-        if type(result_ty) is str:
-            err = "Unknown Z3 error in abstraction (result_ty == '%s'). Update your version of Z3, and, if the problem persists, open a claripy issue." % result_ty
-            l.error(err)
-            raise BackendError(err)
+            if type(result_ty) is str:
+                err = "Unknown Z3 error in abstraction (result_ty == '%s'). Update your version of Z3, and, if the problem persists, open a claripy issue." % result_ty
+                l.error(err)
+                raise BackendError(err)
 
-        if op_name == 'If':
-            # If is polymorphic and thus must be handled specially
-            ty = type(args[1])
+            if op_name == 'If':
+                # If is polymorphic and thus must be handled specially
+                ty = type(args[1])
 
-            a = ty('If', tuple(args), length=args[1].length)
-        elif hasattr(ty, op_name) or hasattr(_all_operations, op_name):
-            op = getattr(ty if hasattr(ty, op_name) else _all_operations, op_name)
-            if op.calc_length is not None:
-                length = op.calc_length(*args)
-                a = result_ty(op_name, tuple(args), length=length)
+                a = ty('If', tuple(args), length=args[1].length)
+            elif hasattr(ty, op_name) or hasattr(_all_operations, op_name):
+                op = getattr(ty if hasattr(ty, op_name) else _all_operations, op_name)
+                if op.calc_length is not None:
+                    length = op.calc_length(*args)
+                    a = result_ty(op_name, tuple(args), length=length)
+                else:
+                    a = result_ty(op_name, tuple(args))
             else:
                 a = result_ty(op_name, tuple(args))
-        else:
-            a = result_ty(op_name, tuple(args))
 
         self._ast_cache[h] = (a, ast)
         z3.Z3_inc_ref(ctx, ast)
@@ -592,15 +588,7 @@ class BackendZ3(Backend):
             return self._abstract_fp_encoded_val(ctx, arg_ast)
         elif op_name == 'INTERNAL' and self._abstract_string_check(ctx, ast):
             # Quirk in how z3 encodes string constants
-            try:
-                assert(z3.Z3_get_decl_num_parameters(ctx, decl) == 1)
-                assert(z3.Z3_get_decl_parameter_kind(ctx, decl, 0) == z3.Z3_PK_SYMBOL)
-                symb = z3.Z3_get_decl_symbol_parameter(ctx, decl, 0)
-                assert(z3.Z3_get_symbol_kind(ctx, symb) == z3.Z3_STRING_SYMBOL)
-                return z3.Z3_get_symbol_string(ctx, symb).encode().decode('unicode_escape')
-                # see https://stackoverflow.com/a/58829514/3154996
-            except AssertionError:
-                raise BackendError("Weird z3 model")
+            return self._string_constant_from_ast(ctx, ast, decl)
         else:
             raise BackendError("Unable to abstract Z3 object to primitive")
 
@@ -691,6 +679,19 @@ class BackendZ3(Backend):
                     return True
         return False
 
+    @staticmethod
+    def _string_constant_from_ast(ctx, ast, decl):
+        # Get Z3-encoded string constants
+        # see https://stackoverflow.com/a/58829514/3154996
+        if z3.Z3_get_decl_num_parameters(ctx, decl) != 1:
+            raise BackendError("Weird Z3 model")
+        if z3.Z3_get_decl_parameter_kind(ctx, decl, 0) != z3.Z3_PK_SYMBOL:
+            raise BackendError("Weird Z3 model")
+        symb = z3.Z3_get_decl_symbol_parameter(ctx, decl, 0)
+        if z3.Z3_get_symbol_kind(ctx, symb) != z3.Z3_STRING_SYMBOL:
+            raise BackendError("Weird Z3 model")
+        return z3.Z3_get_symbol_string(ctx, symb).encode().decode('unicode_escape')
+
     def solver(self, timeout=None):
         if not self.reuse_z3_solver or getattr(self._tls, 'solver', None) is None:
             s = z3.Solver(ctx=self._context)
@@ -773,8 +774,7 @@ class BackendZ3(Backend):
 
         try:
 
-            l.debug("Doing a check!")
-            #print "CHECKING"
+            l.debug("Doing a check! (satisfiable)")
             if solver.check() != z3.sat:
                 return False
 
@@ -807,7 +807,7 @@ class BackendZ3(Backend):
 
         for i in range(n):
             solve_count += 1
-            l.debug("Doing a check!")
+            l.debug("Doing a check! (batch_eval)")
             if solver.check() != z3.sat:
                 break
             model = solver.model()
@@ -866,7 +866,7 @@ class BackendZ3(Backend):
             numpop += 1
 
             solve_count += 1
-            l.debug("Doing a check!")
+            l.debug("Doing a check! (min)")
             if solver.check() == z3.sat:
                 l.debug("... still sat")
                 if model_callback is not None:
@@ -889,7 +889,7 @@ class BackendZ3(Backend):
         else:
             solver.push()
             solver.add(expr == lo)
-            l.debug("Doing a check!")
+            l.debug("Doing a check! (min)")
             if solver.check() == z3.sat:
                 if model_callback is not None:
                     model_callback(self._generic_model(solver.model()))
@@ -931,7 +931,7 @@ class BackendZ3(Backend):
             numpop += 1
 
             solve_count += 1
-            l.debug("Doing a check!")
+            l.debug("Doing a check! (max)")
             if solver.check() == z3.sat:
                 l.debug("... still sat")
                 lo = middle
@@ -953,7 +953,7 @@ class BackendZ3(Backend):
         else:
             solver.push()
             solver.add(expr == hi)
-            l.debug("Doing a check!")
+            l.debug("Doing a check! (max)")
             if solver.check() == z3.sat:
                 if model_callback is not None:
                     model_callback(self._generic_model(solver.model()))
@@ -1071,7 +1071,7 @@ class BackendZ3(Backend):
         return z3.BoolRef(z3.Z3_mk_and(self._context.ref(), sz, _args), self._context)
 
     def _op_raw_Xor(self, *args):
-        return z3.BoolRef(z3.Z3_mk_xor(self._context, *(arg.as_ast() for arg in args)), self._context)
+        return z3.BoolRef(z3.Z3_mk_xor(self._context.ref(), *(arg.as_ast() for arg in args)), self._context)
 
     def _op_raw_Or(self, *args):
         # copied from z3._to_ast_array
@@ -1674,5 +1674,6 @@ op_type_map = {
     'Z3_OP_FPA_MUL': FP,
     'Z3_OP_FPA_DIV': FP,
 
+    'Z3_OP_INTERNAL': None,
     'Z3_OP_UNINTERPRETED': 'UNINTERPRETED'
 }
