@@ -12,7 +12,8 @@ class CompositeFrontend(ConstrainedFrontend):
     def __init__(self, template_frontend, template_frontend_string, track=False, **kwargs):
         super(CompositeFrontend, self).__init__(**kwargs)
         self._solvers = { }
-        self._owned_solvers = weakref.WeakKeyDictionary()
+        self._unchecked_solvers = weakref.WeakSet()
+        self._owned_solvers = weakref.WeakSet()
         self._template_frontend = template_frontend
         self._template_frontend_string = template_frontend_string
         self._unsat = False
@@ -20,7 +21,8 @@ class CompositeFrontend(ConstrainedFrontend):
 
     def _blank_copy(self, c):
         super(CompositeFrontend, self)._blank_copy(c)
-        c._owned_solvers = weakref.WeakKeyDictionary()
+        c._unchecked_solvers = weakref.WeakSet()
+        c._owned_solvers = weakref.WeakSet()
         c._solvers = { }
         c._template_frontend = self._template_frontend
         if hasattr(self, '_template_frontend_string'):
@@ -34,7 +36,8 @@ class CompositeFrontend(ConstrainedFrontend):
         c._track = self._track
 
         c._solvers = dict(self._solvers)
-        self._owned_solvers = weakref.WeakKeyDictionary() # for the COW
+        c._unchecked_solvers = weakref.WeakSet(self._unchecked_solvers)
+        self._owned_solvers = weakref.WeakSet() # for the COW
         return c
 
 
@@ -47,7 +50,8 @@ class CompositeFrontend(ConstrainedFrontend):
 
     def __setstate__(self, s):
         self._solvers, self._template_frontend, self._unsat, self._track, base_state = s
-        self._owned_solvers = weakref.WeakKeyDictionary({s:True for s in self._solver_list})
+        self._owned_solvers = weakref.WeakSet(self._solver_list)
+        self._unchecked_solvers = weakref.WeakSet()
         super().__setstate__(base_state)
 
     def downsize(self):
@@ -70,10 +74,7 @@ class CompositeFrontend(ConstrainedFrontend):
 
     @property
     def variables(self):
-        if len(self._solver_list) == 0:
-            return set()
-        else:
-            return set.union(*[s.variables for s in self._solver_list])
+        return set(self._solvers.keys())
 
     # this is really hacky, but we want to avoid having our variables messed with
     @variables.setter
@@ -163,7 +164,7 @@ class CompositeFrontend(ConstrainedFrontend):
         l.debug("... variable counts: %s", [ len(cs.variables) for cs in ss ])
 
         for ns in ss:
-            self._owned_solvers[ns] = True
+            self._owned_solvers.add(ns)
             self._store_child(ns)
 
         return ss
@@ -181,7 +182,7 @@ class CompositeFrontend(ConstrainedFrontend):
             old_solvers = self._solvers_for_variables(s.variables)
             if len(new_solvers) == len(old_solvers):
                 done = set()
-                for ss in s.split():
+                for ss in new_solvers:
                     if ss in done:
                         continue
                     done.add(ss)
@@ -189,13 +190,15 @@ class CompositeFrontend(ConstrainedFrontend):
                     self._solvers[v].update(ss)
             else:
                 for ns in new_solvers:
-                    self._owned_solvers[ns] = True
+                    self._owned_solvers.add(ns)
                     self._store_child(ns)
 
-    def _store_child(self, ns, extra_names=frozenset()):
+    def _store_child(self, ns, extra_names=frozenset(), invalidate_cache=True):
         for v in ns.variables | extra_names:
             #os = self._solvers[v]
             self._solvers[v] = ns
+        if invalidate_cache:
+            self._unchecked_solvers.add(ns)
 
         #if isinstance(s, ModelCacheMixin):
         #   if len(os._models) < len(ns._models):
@@ -216,7 +219,7 @@ class CompositeFrontend(ConstrainedFrontend):
     def _claim(self, s):
         if s not in self._owned_solvers:
             sc = s.branch()
-            self._owned_solvers[sc] = True
+            self._owned_solvers.add(sc)
             return sc
         else:
             return s
@@ -229,14 +232,14 @@ class CompositeFrontend(ConstrainedFrontend):
         l.debug("Adding %d constraints to %d names", len(constraints), len(names))
         s = self._claim(self._merged_solver_for(names=names))
         added = s.add(constraints, invalidate_cache=invalidate_cache, **kwargs)
-        self._store_child(s)
+        self._store_child(s, invalidate_cache=invalidate_cache)
         return added
 
     def add(self, constraints, **kwargs): #pylint:disable=arguments-differ
         split = self._split_constraints(constraints)
         child_added = [ ]
 
-        l.debug("%s, solvers before: %d", self, len(self._solvers))
+        #l.debug("%s, solvers before: %d", self, len(self._solvers))
         unsure = [ ]
         for names,set_constraints in split:
             if names == { 'CONCRETE' }:
@@ -248,7 +251,7 @@ class CompositeFrontend(ConstrainedFrontend):
             else:
                 child_added += self._add_dependent_constraints(names, set_constraints, **kwargs)
 
-        l.debug("... solvers after add: %d", len(self._solver_list))
+        #l.debug("... solvers after add: %d", len(self._solver_list))
 
         if len(unsure) > 0:
             for s in self._solver_list:
@@ -277,41 +280,27 @@ class CompositeFrontend(ConstrainedFrontend):
             extra_solver_satness = extra_solver.check_satisfiability(extra_constraints=extra_constraints, exact=exact)
             if extra_solver_satness in {'UNSAT', 'UNKNOWN'}:
                 return extra_solver_satness
-
-            satnesses = [
-                s.check_satisfiability(exact=exact) for s in
-                self._solver_list if s.variables.isdisjoint(extra_solver.variables)
-            ]
             self._reabsorb_solver(extra_solver)
-            for satness in satnesses:
-                if satness in {'UNSAT', 'UNKNOWN'}:
-                    return satness
-            return 'SAT'
-        else:
-            for s in self._solver_list:
-                satness = s.check_satisfiability()
-                if satness in {'UNSAT', 'UNKNOWN'}:
-                    return satness
-            return 'SAT'
+
+        for s in self._unchecked_solvers:
+            if extra_constraints and s.variables & extra_solver.variables:
+                # skip solvers covered by extra constraints (they were checked above)
+                continue
+
+            if self._solvers[min(iter(s.variables))] is not s:
+                # this happens when a parent solver didn't check all unchecked solvers, and we have stale
+                # child solvers in the unchecked list
+                continue
+
+            satness = s.check_satisfiability(exact=exact)
+            if satness in {'UNSAT', 'UNKNOWN'}:
+                return satness
+
+        self._unchecked_solvers.clear()
+        return 'SAT'
 
     def satisfiable(self, extra_constraints=(), exact=None):
-        if self._unsat: return False
-
-        l.debug("%r checking satisfiability...", self)
-
-        if len(extra_constraints) != 0:
-            extra_solver = self._merged_solver_for(lst=extra_constraints)
-            if not extra_solver.satisfiable(extra_constraints=extra_constraints, exact=exact):
-                return False
-
-            r = all(
-                s.satisfiable(exact=exact) for s in
-                self._solver_list if s.variables.isdisjoint(extra_solver.variables)
-            )
-            self._reabsorb_solver(extra_solver)
-            return r
-        else:
-            return all(s.satisfiable(exact=exact) for s in self._solver_list)
+        return self.check_satisfiability(extra_constraints=extra_constraints, exact=exact) == 'SAT'
 
     def eval(self, e, n, extra_constraints=(), exact=None):
         self._ensure_sat(extra_constraints=extra_constraints)
@@ -376,7 +365,7 @@ class CompositeFrontend(ConstrainedFrontend):
         cores = [ ]
 
         for solver in self._solver_list:
-            cores.extend(list(solver.unsat_core()))
+            cores.extend(list(solver.unsat_core(extra_constraints=extra_constraints)))
 
         return cores
 
@@ -441,9 +430,9 @@ class CompositeFrontend(ConstrainedFrontend):
         l.debug("... %s common solvers", len(common_solvers))
 
         for s in common_solvers:
-            self._owned_solvers.pop(s, None)
+            self._owned_solvers.discard(s)
             for o in others:
-                o._owned_solvers.pop(s, None)
+                o._owned_solvers.discard(s)
 
             for v in s.variables:
                 merged._solvers[v] = s
@@ -466,7 +455,7 @@ class CompositeFrontend(ConstrainedFrontend):
                 combined_noncommons[1:], merge_conditions
             )
 
-            merged._owned_solvers[merged_noncommon] = True
+            merged._owned_solvers.add(merged_noncommon)
             merged._store_child(merged_noncommon)
 
         merged.constraints = list(
